package com.qy.worksheetsystem.model.mapper;

import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

@Repository
public interface WorkSheetMapper {
    /**
     * 通过审核人获取明细信息
     * @param id
     * @return
     */
    List<Map<String,String>> getReportDateWorkHoursByPerson(@Param("personId")String personId,@Param("reportDate")String reportDate,@Param("id")String id);


    /**
     * 通过审核人获取明细信息
     * @param id
     * @return
     */
    List<Map<String,String>> getEntryByApproverId(@Param("id")String id);

    /**
     * 通过parentid获取明细
     * @param id
     * @return
     */
    List<Map<String,String>> getEntryByParentId(@Param("id")String id);

    /**
     * 获取明细数据
     * @param id
     * @return
     */
    Map<String,String> getEntryById(@Param("id")String id);
    /**
     * 获取明细数据
     * @param id
     * @return
     */
    Map<String,Object> getJobBookingById(@Param("id")String id);
    /**
     * 更新明细表状态
     * @param status
     * @param id
     * @return
     */
    public boolean updateEntryStatusById(@Param("status") Integer status,@Param("id") String id);
    /**
     * 更新主表状态
     * @param status
     * @param id
     * @return
     */
    public boolean updateStatusById(@Param("status") Integer status,@Param("id") String id);

    /**
     * 更新主表状态
     * @param map

     * @return
     */
    public boolean updateJobBooking(Map map);

    /**
     * 更新主表状态
     * @param map
     * @return
     */
    public boolean updateEntry(Map map);
    /**
     * 通过主表id更新明细状态
     * @param status
     * @param parentId
     * @return
     */
    public boolean updateEntryStatusByParentId(@Param("status") Integer status,@Param("id") String parentId);

    /**
     * 根据id更新parentId
     * @param parentId
     * @param id
     * @return
     */
    public boolean updateEntryParentIdById(@Param("parentId") String parentId,@Param("id") String id);


    public boolean updateEntryAudiByParentId(@Param("status") String status,@Param("id") String id,
                                             @Param("auditorAndNumber") String auditorAndNumber);
    public boolean updateStatusAndAuditorById(@Param("status") String status,@Param("id") String id,
                                              @Param("auditorAndNumber") String auditorAndNumber);
}
